/*
 	File:		AppleTalk.h
 
 	Contains:	AppleTalk Interfaces.
 
 	Version:	Technology:	System 7.5
 				Release:	Universal Interfaces 3.0d3 on Copland DR1
 
 	Copyright:	 1984-1996 by Apple Computer, Inc.  All rights reserved.
 
 	Bugs?:		If you find a problem with this file, send the file and version
 				information (from above) and the problem description to:
 
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/
#ifndef __APPLETALK__
#define __APPLETALK__

#ifndef __TYPES__
#include <Types.h>
#endif
#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

#if FOR_SYSTEM7_AND_SYSTEM8_COOPERATIVE
/*
	Real definition of EntityName is 3 PACKED strings of any length (32 is just an example). No
	offests for Asm since each String address must be calculated by adding length byte to last string ptr.
	In Pascal, String(32) will be 34 bytes long since fields never start on an odd byte unless they are 
	only a byte long. So this will generate correct looking interfaces for Pascal and C, but they will not
	be the same, which is OK since they are not used. 
*/
struct EntityName {
	Str32Field 						objStr;
	Str32Field 						typeStr;
	Str32Field 						zoneStr;
};
typedef struct EntityName EntityName;

typedef EntityName *EntityPtr;
struct AddrBlock {
	UInt16 							aNet;
	UInt8 							aNode;
	UInt8 							aSocket;
};
typedef struct AddrBlock AddrBlock;

#endif
#if FOR_SYSTEM7_AND_SYSTEM8_DEPRECATED

enum {
																/* Driver unit and reference numbers (ADSP is dynamic) */
	mppUnitNum					= 9,							/* MPP unit number */
	atpUnitNum					= 10,							/* ATP unit number */
	xppUnitNum					= 40,							/* XPP unit number */
	mppRefNum					= -10,							/* MPP reference number */
	atpRefNum					= -11,							/* ATP reference number */
	xppRefNum					= -41,							/* XPP reference number */
																/* .MPP csCodes */
	lookupReply					= 242,							/* This command queued to ourself */
	writeLAP					= 243,							/* Write out LAP packet */
	detachPH					= 244,							/* Detach LAP protocol handler */
	attachPH					= 245,							/* Attach LAP protocol handler */
	writeDDP					= 246,							/* Write out DDP packet */
	closeSkt					= 247,							/* Close DDP socket */
	openSkt						= 248,							/* Open DDP socket */
	loadNBP						= 249,							/* Load NBP command-executing code */
	lastResident				= 249,							/* Last resident command */
	confirmName					= 250,							/* Confirm name */
	lookupName					= 251,							/* Look up name on internet */
	removeName					= 252,							/* Remove name from Names Table */
	registerName				= 253,							/* Register name in Names Table */
	killNBP						= 254							/* Kill outstanding NBP request */
};


enum {
	unloadNBP					= 255,							/* Unload NBP command code */
	setSelfSend					= 256,							/* MPP: Set to allow writes to self */
	SetMyZone					= 257,							/* Set my zone name */
	GetATalkInfo				= 258,							/* get AppleTalk information */
	ATalkClosePrep				= 259,							/* AppleTalk close query */
																/* .ATP csCodes */
	nSendRequest				= 248,							/* NSendRequest code */
	relRspCB					= 249,							/* Release RspCB */
	closeATPSkt					= 250,							/* Close ATP socket */
	addResponse					= 251,							/* Add response code | Require open skt */
	sendResponse				= 252,							/* Send response code */
	getRequest					= 253,							/* Get request code */
	openATPSkt					= 254,							/* Open ATP socket */
	sendRequest					= 255,							/* Send request code */
	relTCB						= 256,							/* Release TCB */
	killGetReq					= 257,							/* Kill GetRequest */
	killSendReq					= 258,							/* Kill SendRequest */
	killAllGetReq				= 259,							/* Kill all getRequests for a skt */
																/* .XPP csCodes */
	openSess					= 255,							/* Open session */
	closeSess					= 254,							/* Close session */
	userCommand					= 253							/* User command */
};


enum {
	userWrite					= 252,							/* User write */
	getStatus					= 251,							/* Get status */
	afpCall						= 250,							/* AFP command (buffer has command code) */
	getParms					= 249,							/* Get parameters */
	abortOS						= 248,							/* Abort open session request */
	closeAll					= 247,							/* Close all open sessions */
	xCall						= 246,							/* .XPP extended calls */
																/* Transition Queue transition types */
	ATTransOpen					= 0,							/*AppleTalk has opened*/
	ATTransClose				= 2,							/*AppleTalk is about to close*/
	ATTransClosePrep			= 3,							/*Is it OK to close AppleTalk ?*/
	ATTransCancelClose			= 4,							/*Cancel the ClosePrep transition*/
	afpByteRangeLock			= 1,							/*AFPCall command codes*/
	afpVolClose					= 2,							/*AFPCall command codes*/
	afpDirClose					= 3,							/*AFPCall command codes*/
	afpForkClose				= 4,							/*AFPCall command codes*/
	afpCopyFile					= 5,							/*AFPCall command codes*/
	afpDirCreate				= 6,							/*AFPCall command codes*/
	afpFileCreate				= 7,							/*AFPCall command codes*/
	afpDelete					= 8,							/*AFPCall command codes*/
	afpEnumerate				= 9								/*AFPCall command codes*/
};


enum {
	afpFlush					= 10,							/*AFPCall command codes*/
	afpForkFlush				= 11,							/*AFPCall command codes*/
	afpGetDirParms				= 12,							/*AFPCall command codes*/
	afpGetFileParms				= 13,							/*AFPCall command codes*/
	afpGetForkParms				= 14,							/*AFPCall command codes*/
	afpGetSInfo					= 15,							/*AFPCall command codes*/
	afpGetSParms				= 16,							/*AFPCall command codes*/
	afpGetVolParms				= 17,							/*AFPCall command codes*/
	afpLogin					= 18,							/*AFPCall command codes*/
	afpContLogin				= 19,							/*AFPCall command codes*/
	afpLogout					= 20,							/*AFPCall command codes*/
	afpMapID					= 21,							/*AFPCall command codes*/
	afpMapName					= 22,							/*AFPCall command codes*/
	afpMove						= 23,							/*AFPCall command codes*/
	afpOpenVol					= 24,							/*AFPCall command codes*/
	afpOpenDir					= 25,							/*AFPCall command codes*/
	afpOpenFork					= 26,							/*AFPCall command codes*/
	afpRead						= 27,							/*AFPCall command codes*/
	afpRename					= 28,							/*AFPCall command codes*/
	afpSetDirParms				= 29							/*AFPCall command codes*/
};


enum {
	afpSetFileParms				= 30,							/*AFPCall command codes*/
	afpSetForkParms				= 31,							/*AFPCall command codes*/
	afpSetVolParms				= 32,							/*AFPCall command codes*/
	afpWrite					= 33,							/*AFPCall command codes*/
	afpGetFlDrParms				= 34,							/*AFPCall command codes*/
	afpSetFlDrParms				= 35,							/*AFPCall command codes*/
	afpDTOpen					= 48,							/*AFPCall command codes*/
	afpDTClose					= 49,							/*AFPCall command codes*/
	afpGetIcon					= 51,							/*AFPCall command codes*/
	afpGtIcnInfo				= 52,							/*AFPCall command codes*/
	afpAddAPPL					= 53,							/*AFPCall command codes*/
	afpRmvAPPL					= 54,							/*AFPCall command codes*/
	afpGetAPPL					= 55,							/*AFPCall command codes*/
	afpAddCmt					= 56,							/*AFPCall command codes*/
	afpRmvCmt					= 57,							/*AFPCall command codes*/
	afpGetCmt					= 58,							/*AFPCall command codes*/
	afpAddIcon					= 192,							/*Special code for ASP Write commands*/
	xppLoadedBit				= 5,							/* XPP bit in PortBUse */
	scbMemSize					= 192,							/* Size of memory for SCB */
	xppFlagClr					= 0								/* Cs for AFPCommandBlock */
};


enum {
	xppFlagSet					= 128,							/* StartEndFlag & NewLineFlag fields. */
	lapSize						= 20,
	ddpSize						= 26,
	nbpSize						= 26,
	atpSize						= 56,
	atpXOvalue					= 32,							/*ATP exactly-once bit */
	atpEOMvalue					= 16,							/*ATP End-Of-Message bit */
	atpSTSvalue					= 8,							/*ATP Send-Transmission-Status bit */
	atpTIDValidvalue			= 2,							/*ATP trans. ID valid bit */
	atpSendChkvalue				= 1,							/*ATP send checksum bit */
	zipGetLocalZones			= 5,
	zipGetZoneList				= 6,
	zipGetMyZone				= 7,
	LAPMgrPtr					= 0x0B18,						/*Entry point for LAP Manager*/
	LAPMgrCall					= 2,							/*Offset to LAP routines*/
	LAddAEQ						= 23,							/*LAPAddATQ routine selector*/
	LRmvAEQ						= 24							/*LAPRmvATQ routine selector*/
};

#define MPPioCompletion MPP.ioCompletion
#define MPPioResult MPP.ioResult
#define MPPioRefNum MPP.ioRefNum
#define MPPcsCode MPP.csCode
#define LAPprotType LAP.protType
#define LAPwdsPointer LAP.u.wdsPointer
#define LAPhandler LAP.u.handler
#define DDPsocket DDP.socket
#define DDPchecksumFlag DDP.checksumFlag
#define DDPwdsPointer DDP.u.wdsPointer
#define DDPlistener DDP.u.listener
#define NBPinterval NBP.interval
#define NBPcount NBP.count
#define NBPntQElPtr NBP.nbpPtrs.ntQElPtr
#define NBPentityPtr NBP.nbpPtrs.entityPtr
#define NBPverifyFlag NBP.parm.verifyFlag
#define NBPretBuffPtr NBP.parm.Lookup.retBuffPtr
#define NBPretBuffSize NBP.parm.Lookup.retBuffSize
#define NBPmaxToGet NBP.parm.Lookup.maxToGet
#define NBPnumGotten NBP.parm.Lookup.numGotten
#define NBPconfirmAddr NBP.parm.Confirm.confirmAddr
#define NBPnKillQEl NBPKILL.nKillQEl
#define NBPnewSocket NBP.parm.Confirm.newSocket
#define ATPioCompletion ATP.ioCompletion
#define ATPioResult ATP.ioResult
#define ATPuserData ATP.userData
#define ATPreqTID ATP.reqTID
#define ATPioRefNum ATP.ioRefNum
#define ATPcsCode ATP.csCode
#define ATPatpSocket ATP.atpSocket
#define ATPatpFlags ATP.atpFlags
#define ATPaddrBlock ATP.addrBlock
#define ATPreqLength ATP.reqLength
#define ATPreqPointer ATP.reqPointer
#define ATPbdsPointer ATP.bdsPointer
#define ATPtimeOutVal SREQ.timeOutVal
#define ATPnumOfResps SREQ.numOfResps
#define ATPretryCount SREQ.retryCount
#define ATPnumOfBuffs OTH1.u.numOfBuffs
#define ATPbitMap OTH1.u.bitMap
#define ATPrspNum OTH1.u.rspNum
#define ATPbdsSize OTH2.bdsSize
#define ATPtransID OTH2.transID
#define ATPaKillQEl KILL.aKillQEl

enum {
	tLAPRead					= 0,
	tLAPWrite					= 1,
	tDDPRead					= 2,
	tDDPWrite					= 3,
	tNBPLookup					= 4,
	tNBPConfirm					= 5,
	tNBPRegister				= 6,
	tATPSndRequest				= 7,
	tATPGetRequest				= 8,
	tATPSdRsp					= 9,
	tATPAddRsp					= 10,
	tATPRequest					= 11,
	tATPResponse				= 12
};

typedef SInt8 ABCallType;

enum {
	lapProto					= 0,
	ddpProto					= 1,
	nbpProto					= 2,
	atpProto					= 3
};

typedef UInt8 ABProtoType;
typedef Byte ABByte;
struct LAPAdrBlock {
	UInt8 							dstNodeID;
	UInt8 							srcNodeID;
	ABByte 							lapProtType;
	UInt8 							filler;						/*	Filler for proper byte alignment*/
};
typedef struct LAPAdrBlock LAPAdrBlock;

typedef struct ATQEntry ATQEntry;
typedef ATQEntry *ATQEntryPtr;
typedef long (*ATalkTransitionEventProcPtr)(long eventCode, ATQEntryPtr qElem, void *eventParameter);

#if GENERATINGCFM
typedef UniversalProcPtr ATalkTransitionEventUPP;
#else
typedef ATalkTransitionEventProcPtr ATalkTransitionEventUPP;
#endif
typedef ATalkTransitionEventUPP ATalkTransitionEvent;
struct ATQEntry {
	struct ATQEntry *				qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	ATalkTransitionEventUPP 		CallAddr;					/*your routine descriptor*/
};

struct RetransType {
	UInt8 							retransInterval;
	UInt8 							retransCount;
};
typedef struct RetransType RetransType;

struct BDSElement {
	short 							buffSize;
	Ptr 							buffPtr;
	short 							dataSize;
	long 							userBytes;
};
typedef struct BDSElement BDSElement;

typedef BDSElement BDSType[8];
typedef BDSElement *BDSPtr;
typedef char BitMapType;
struct ATLAPRec {
	ABCallType 						abOpcode;
	SInt8 							filler;						/*	Filler for proper byte alignment*/
	short 							abResult;
	long 							abUserReference;
	LAPAdrBlock 					lapAddress;
	short 							lapReqCount;
	short 							lapActCount;
	Ptr 							lapDataPtr;
};
typedef struct ATLAPRec ATLAPRec;

typedef ATLAPRec *ATLAPRecPtr;
typedef ATLAPRecPtr *ATLAPRecHandle;
struct ATDDPRec {
	ABCallType 						abOpcode;
	SInt8 							filler;						/*	Filler for proper byte alignment*/
	short 							abResult;
	long 							abUserReference;
	short 							ddpType;
	short 							ddpSocket;
	AddrBlock 						ddpAddress;
	short 							ddpReqCount;
	short 							ddpActCount;
	Ptr 							ddpDataPtr;
	short 							ddpNodeID;
};
typedef struct ATDDPRec ATDDPRec;

typedef ATDDPRec *ATDDPRecPtr;
typedef ATDDPRecPtr *ATDDPRecHandle;
struct ATNBPRec {
	ABCallType 						abOpcode;
	SInt8 							filler;						/*	Filler for proper byte alignment*/
	short 							abResult;
	long 							abUserReference;
	EntityPtr 						nbpEntityPtr;
	Ptr 							nbpBufPtr;
	short 							nbpBufSize;
	short 							nbpDataField;
	AddrBlock 						nbpAddress;
	RetransType 					nbpRetransmitInfo;
};
typedef struct ATNBPRec ATNBPRec;

typedef ATNBPRec *ATNBPRecPtr;
typedef ATNBPRecPtr *ATNBPRecHandle;
struct ATATPRec {
	ABCallType 						abOpcode;
	SInt8 							filler1;					/*	Filler for proper byte alignment*/
	short 							abResult;
	long 							abUserReference;
	short 							atpSocket;
	AddrBlock 						atpAddress;
	short 							atpReqCount;
	Ptr 							atpDataPtr;
	BDSPtr 							atpRspBDSPtr;
	BitMapType 						atpBitMap;
	SInt8 							filler2;					/*	Filler for proper byte alignment*/
	short 							atpTransID;
	short 							atpActCount;
	long 							atpUserData;
	Boolean 						atpXO;
	Boolean 						atpEOM;
	short 							atpTimeOut;
	short 							atpRetries;
	short 							atpNumBufs;
	short 							atpNumRsp;
	short 							atpBDSSize;
	long 							atpRspUData;
	Ptr 							atpRspBuf;
	short 							atpRspSize;
};
typedef struct ATATPRec ATATPRec;

typedef ATATPRec *ATATPRecPtr;
typedef ATATPRecPtr *ATATPRecHandle;
struct AFPCommandBlock {
	UInt8 							cmdByte;
	UInt8 							startEndFlag;
	short 							forkRefNum;
	long 							rwOffset;
	long 							reqCount;
	UInt8 							newLineFlag;
	char 							newLineChar;
};
typedef struct AFPCommandBlock AFPCommandBlock;

typedef union MPPParamBlock MPPParamBlock;
typedef union ATPParamBlock ATPParamBlock;
typedef union XPPParamBlock XPPParamBlock;
typedef MPPParamBlock *MPPPBPtr;
typedef ATPParamBlock *ATPPBPtr;
typedef XPPParamBlock *XPPParmBlkPtr;
/*
		This ProcPtr uses register based parameters on the 68k and cannot
		be written in or called from a high-level language without the help of
		mixed mode or assembly glue.

			typedef pascal void (*MPPCompletionProcPtr)(MPPPBPtr thePBptr);

*/
/*
		This ProcPtr uses register based parameters on the 68k and cannot
		be written in or called from a high-level language without the help of
		mixed mode or assembly glue.

			typedef pascal void (*ATPCompletionProcPtr)(ATPPBPtr thePBptr);

*/
/*
		This ProcPtr uses register based parameters on the 68k and cannot
		be written in or called from a high-level language without the help of
		mixed mode or assembly glue.

			typedef pascal void (*XPPCompletionProcPtr)(XPPParmBlkPtr thePBptr);

*/
/*
		This ProcPtr uses register based parameters on the 68k and cannot
		be written in or called from a high-level language without the help of
		mixed mode or assembly glue.

			typedef pascal void (*AttnRoutineProcPtr)(short sessRefnum, short attnBytes);

*/

#if GENERATINGCFM
typedef UniversalProcPtr MPPCompletionUPP;
typedef UniversalProcPtr ATPCompletionUPP;
typedef UniversalProcPtr XPPCompletionUPP;
typedef UniversalProcPtr AttnRoutineUPP;
#else
typedef Register68kProcPtr MPPCompletionUPP;
typedef Register68kProcPtr ATPCompletionUPP;
typedef Register68kProcPtr XPPCompletionUPP;
typedef Register68kProcPtr AttnRoutineUPP;
#endif
struct WDSElement {
	short 							entryLength;
	Ptr 							entryPtr;
};
typedef struct WDSElement WDSElement;

struct NTElement {
	AddrBlock 						nteAddress;					/*network address of entity*/
	SInt8 							filler;
	SInt8 							entityData[99];				/*Object, Type & Zone*/
};
typedef struct NTElement NTElement;

struct NamesTableEntry {
	Ptr 							qNext;						/*ptr to next NTE*/
	NTElement 						nt;
};
typedef struct NamesTableEntry NamesTableEntry;

/*
		This ProcPtr uses register based parameters on the 68k and cannot
		be written in or called from a high-level language without the help of
		mixed mode or assembly glue.

			typedef pascal Boolean (*MPPProtocolHandlerProcPtr)(Ptr SCCAddr1, Ptr SCCAddr2, Ptr MPPLocalVars, Ptr nextFreeByteInRHA, Ptr ReadPacketAndReadRestPtr, short numBytesLeftToReadInPacket);

*/
/*
		This ProcPtr uses register based parameters on the 68k and cannot
		be written in or called from a high-level language without the help of
		mixed mode or assembly glue.

			typedef pascal Boolean (*DDPSocketListenerProcPtr)(Ptr SCCAddr1, Ptr SCCAddr2, Ptr MPPLocalVars, Ptr nextFreeByteInRHA, Ptr ReadPacketAndReadRestPtr, UInt8 packetDestinationNumber, short numBytesLeftToReadInPacket);

*/

#if GENERATINGCFM
typedef UniversalProcPtr MPPProtocolHandlerUPP;
typedef UniversalProcPtr DDPSocketListenerUPP;
#else
typedef Register68kProcPtr MPPProtocolHandlerUPP;
typedef Register68kProcPtr DDPSocketListenerUPP;
#endif
/*
	MPPProtocolHandlerProcs and  DDPSocketListenerProcs cannot be written 
	in or called from a high-level language without the help of mixed mode 
	or assembly glue because they use the following parameter-passing conventions:

	typedef Boolean (*MPPProtocolHandlerProcPtr)(Ptr SCCAddr1, Ptr SCCAddr2, 
			Ptr MPPLocalVars, Ptr nextFreeByteInRHA, Ptr ReadPacketAndReadRestPtr, 
			short numBytesLeftToReadInPacket);

		In:
			=>	SCCAddr1					A0.L
			=>	SCCAddr2					A1.L
			=>	MPPLocalVars				A2.L
			=>	nextFreeByteInRHA			A3.L
			=>	ReadPacketAndReadRestPtr	A4.L
			=>	numBytesLeftToReadInPacket	D1.W
		Out:
			<=	Boolean						Z bit of CCR

	typedef Boolean (*DDPSocketListenerProcPtr)(Ptr SCCAddr1, Ptr SCCAddr2, 
			Ptr MPPLocalVars, Ptr nextFreeByteInRHA, Ptr ReadPacketAndReadRestPtr, 
			UInt8 packetDestinationNumber, short numBytesLeftToReadInPacket);

		In:
			=>	SCCAddr1					A0.L
			=>	SCCAddr2					A1.L
			=>	MPPLocalVars				A2.L
			=>	nextFreeByteInRHA			A3.L
			=>	ReadPacketAndReadRestPtr	A4.L
			=>	packetDestinationNumber		D0.B
			=>	numBytesLeftToReadInPacket	D1.W
		Out:
			<=	Boolean						Z bit of CCR

*/
struct MPPparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
};
typedef struct MPPparms MPPparms;

struct LAPparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							protType;					/*ALAP protocol Type */
	UInt8 							filler;
	union {
		Ptr 							wdsPointer;				/*-> write data structure*/
		MPPProtocolHandlerUPP 			handler;				/*-> protocol handler routine*/
	} 								u;
};
typedef struct LAPparms LAPparms;

struct DDPparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							socket;						/*socket number */
	UInt8 							checksumFlag;				/*check sum flag */
	union {
		Ptr 							wdsPointer;				/*-> write data structure*/
		DDPSocketListenerUPP 			listener;				/*->write data structure or -> Listener*/
	} 								u;
};
typedef struct DDPparms DDPparms;

union NBPPtrs {
	Ptr 							ntQElPtr;
	Ptr 							entityPtr;
};
typedef union NBPPtrs NBPPtrs;

union LookupConfirmParams {
	UInt8 							verifyFlag;
	struct {
		Ptr 							retBuffPtr;
		short 							retBuffSize;
		short 							maxToGet;
		short 							numGotten;
	} 								Lookup;
	struct {
		AddrBlock 						confirmAddr;
		UInt8 							newSocket;
		SInt8 							filler;					/*	Filler for proper byte alignment*/
	} 								Confirm;
};
typedef union LookupConfirmParams LookupConfirmParams;

struct NBPparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							interval;					/*retry interval */
	UInt8 							count;						/*retry count */
	NBPPtrs 						nbpPtrs;
	LookupConfirmParams 			parm;
};
typedef struct NBPparms NBPparms;

struct SetSelfparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							newSelfFlag;				/*self-send toggle flag */
	UInt8 							oldSelfFlag;				/*previous self-send state */
};
typedef struct SetSelfparms SetSelfparms;

struct NBPKillparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	Ptr 							nKillQEl;					/*ptr to i/o queue element to cancel */
};
typedef struct NBPKillparms NBPKillparms;

struct GetAppleTalkInfoParm {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
																/*max. concurrent NBP requests*/
	short 							version;					/*requested info version*/
	Ptr 							varsPtr;					/*pointer to well known MPP vars*/
	Ptr 							DCEPtr;						/*pointer to MPP DCE*/
	short 							portID;						/*port number [0..7]*/
	long 							configuration;				/*32-bit configuration word*/
	short 							selfSend;					/*non zero if SelfSend enabled*/
	short 							netLo;						/*low value of network range*/
	short 							netHi;						/*high value of network range*/
	long 							ourAdd;						/*our 24-bit AppleTalk address*/
	long 							routerAddr;					/*24-bit address of (last) router*/
	short 							numOfPHs;					/*max. number of protocol handlers*/
	short 							numOfSkts;					/*max. number of static sockets*/
	short 							numNBPEs;					/*max. concurrent NBP requests*/
	Ptr 							nTQueue;					/*pointer to registered name queue*/
	short 							LAlength;					/*length in bytes of data link addr*/
	Ptr 							linkAddr;					/*data link address returned*/
	Ptr 							zoneName;					/*zone name returned*/
};
typedef struct GetAppleTalkInfoParm GetAppleTalkInfoParm;

struct ATalkClosePrepParm {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	Ptr 							appName;					/*pointer to application name in buffer*/
};
typedef struct ATalkClosePrepParm ATalkClosePrepParm;

union MPPParamBlock {
	MPPparms 						MPP;						/*General MPP parms*/
	LAPparms 						LAP;						/*ALAP calls*/
	DDPparms 						DDP;						/*DDP calls*/
	NBPparms 						NBP;						/*NBP calls*/
	SetSelfparms 					SETSELF;
	NBPKillparms 					NBPKILL;
	GetAppleTalkInfoParm 			GAIINFO;
	ATalkClosePrepParm 				ATALKCLOSE;
};

struct XPPPrmBlk {
	QElem *							qLink;
	short 							qType;
	short 							ioTrap;
	Ptr 							ioCmdAddr;
	XPPCompletionUPP 				ioCompletion;
	OSErr 							ioResult;
	long 							cmdResult;
	short 							ioVRefNum;
	short 							ioRefNum;
	short 							csCode;
	short 							sessRefnum;					/*Offset to session refnum */
	UInt8 							aspTimeout;					/*Timeout for ATP */
	UInt8 							aspRetry;					/*Retry count for ATP */
	short 							cbSize;						/*Command block size */
	Ptr 							cbPtr;						/*Command block pointer */
	short 							rbSize;						/*Reply buffer size */
	Ptr 							rbPtr;						/*Reply buffer pointer */
	short 							wdSize;						/*Write Data size*/
	Ptr 							wdPtr;						/*Write Data pointer*/
	UInt8 							ccbStart[296];				/*CCB memory allocated for driver afpWrite max size(CCB)=296 all other calls=150*/
};
typedef struct XPPPrmBlk XPPPrmBlk;

struct ASPGetparmsBlk {
	QElem *							qLink;
	short 							qType;
	short 							ioTrap;
	Ptr 							ioCmdAddr;
	XPPCompletionUPP 				ioCompletion;
	OSErr 							ioResult;
	long 							cmdResult;
	short 							ioVRefNum;
	short 							ioRefNum;
	short 							csCode;
	short 							aspMaxCmdSize;				/*For SPGetParms*/
	short 							aspQuantumSize;
	short 							numSesss;
};
typedef struct ASPGetparmsBlk ASPGetparmsBlk;

struct ASPAbortPrm {
	QElem *							qLink;
	short 							qType;
	short 							ioTrap;
	Ptr 							ioCmdAddr;
	XPPCompletionUPP 				ioCompletion;
	OSErr 							ioResult;
	long 							cmdResult;
	short 							ioVRefNum;
	short 							ioRefNum;
	short 							csCode;
	Ptr 							abortSCBPtr;				/*SCB pointer for AbortOS */
};
typedef struct ASPAbortPrm ASPAbortPrm;

struct ASPOpenPrm {
	QElem *							qLink;
	short 							qType;
	short 							ioTrap;
	Ptr 							ioCmdAddr;
	XPPCompletionUPP 				ioCompletion;
	OSErr 							ioResult;
	long 							cmdResult;
	short 							ioVRefNum;
	short 							ioRefNum;
	short 							csCode;
	short 							sessRefnum;					/*Offset to session refnum */
	UInt8 							aspTimeout;					/*Timeout for ATP */
	UInt8 							aspRetry;					/*Retry count for ATP */
	AddrBlock 						serverAddr;					/*Server address block */
	Ptr 							scbPointer;					/*SCB pointer */
	AttnRoutineUPP 					attnRoutine;				/*Attention routine pointer*/
};
typedef struct ASPOpenPrm ASPOpenPrm;

typedef ASPOpenPrm *ASPOpenPrmPtr;
struct AFPLoginPrm {
	QElem *							qLink;
	short 							qType;
	short 							ioTrap;
	Ptr 							ioCmdAddr;
	XPPCompletionUPP 				ioCompletion;
	OSErr 							ioResult;
	long 							cmdResult;
	short 							ioVRefNum;
	short 							ioRefNum;
	short 							csCode;
	short 							sessRefnum;					/*Offset to session refnum */
	UInt8 							aspTimeout;					/*Timeout for ATP */
	UInt8 							aspRetry;					/*Retry count for ATP */
	short 							cbSize;						/*Command block size */
	Ptr 							cbPtr;						/*Command block pointer */
	short 							rbSize;						/*Reply buffer size */
	Ptr 							rbPtr;						/*Reply buffer pointer */
	AddrBlock 						afpAddrBlock;				/*block in AFP login */
	Ptr 							afpSCBPtr;					/*SCB pointer in AFP login */
	Ptr 							afpAttnRoutine;				/*routine pointer in AFP login */
	UInt8 							ccbFill[144];				/*CCB memory allocated for driver  Login needs only 150 bytes BUT CCB really starts in the middle of AFPSCBPtr and also clobbers AFPAttnRoutine. */
};
typedef struct AFPLoginPrm AFPLoginPrm;

struct XCallParam {
	QElem *							qLink;
	short 							qType;
	short 							ioTrap;
	Ptr 							ioCmdAddr;
	XPPCompletionUPP 				ioCompletion;
	OSErr 							ioResult;
	long 							cmdResult;
	short 							ioVRefNum;
	short 							ioRefNum;
	short 							csCode;
	short 							xppSubCode;
	UInt8 							xppTimeout;					/*retry interval (seconds)*/
	UInt8 							xppRetry;					/*retry count*/
	short 							filler1;
	Ptr 							zipBuffPtr;					/*pointer to buffer (must be 578 bytes)*/
	short 							zipNumZones;				/*no. of zone names in this response*/
	UInt8 							zipLastFlag;				/*non-zero if no more zones*/
	UInt8 							filler2;					/*filler*/
	UInt8 							zipInfoField[70];			/*on initial call, set first word to zero*/
};
typedef struct XCallParam XCallParam;

union XPPParamBlock {
	XPPPrmBlk 						XPP;
	ASPGetparmsBlk 					GETPARM;
	ASPAbortPrm 					ABORT;
	ASPOpenPrm 						OPEN;
	AFPLoginPrm 					LOGIN;
	XCallParam 						XCALL;
};

struct ATPparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							atpSocket;					/*currbitmap for requests or ATP socket number*/
	UInt8 							atpFlags;					/*control information*/
	AddrBlock 						addrBlock;					/*source/dest. socket address*/
	short 							reqLength;					/*request/response length*/
	Ptr 							reqPointer;					/*->request/response Data*/
	Ptr 							bdsPointer;					/*->response BDS */
};
typedef struct ATPparms ATPparms;

struct SendReqparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							atpSocket;					/*currbitmap for requests or ATP socket number*/
	UInt8 							atpFlags;					/*control information*/
	AddrBlock 						addrBlock;					/*source/dest. socket address*/
	short 							reqLength;					/*request/response length*/
	Ptr 							reqPointer;					/*->request/response Data*/
	Ptr 							bdsPointer;					/*->response BDS */
	UInt8 							numOfBuffs;					/*numOfBuffs */
	UInt8 							timeOutVal;					/*timeout interval */
	UInt8 							numOfResps;					/*number of responses actually received */
	UInt8 							retryCount;					/*number of retries */
	short 							intBuff;					/*used internally for NSendRequest */
	UInt8 							TRelTime;					/*TRelease time for extended send request */
	SInt8 							filler0;
};
typedef struct SendReqparms SendReqparms;

struct ATPmisc1 {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							atpSocket;					/*currbitmap for requests or ATP socket number*/
	UInt8 							atpFlags;					/*control information*/
	AddrBlock 						addrBlock;					/*source/dest. socket address*/
	short 							reqLength;					/*request/response length*/
	Ptr 							reqPointer;					/*->request/response Data*/
	Ptr 							bdsPointer;					/*->response BDS */
	union {
		UInt8 							bitMap;					/*bitmap received */
		UInt8 							numOfBuffs;				/*number of responses being sent*/
		UInt8 							rspNum;					/*sequence number*/
	} 								u;
};
typedef struct ATPmisc1 ATPmisc1;

struct ATPmisc2 {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							atpSocket;					/*currbitmap for requests or ATP socket number*/
	UInt8 							atpFlags;					/*control information*/
	AddrBlock 						addrBlock;					/*source/dest. socket address*/
	short 							reqLength;					/*request/response length*/
	Ptr 							reqPointer;					/*->request/response Data*/
	Ptr 							bdsPointer;					/*->response BDS */
	UInt8 							filler;
	UInt8 							bdsSize;					/*number of BDS elements */
	short 							transID;					/*transaction ID recd. */
};
typedef struct ATPmisc2 ATPmisc2;

struct Killparms {
	QElem *							qLink;						/*next queue entry*/
	short 							qType;						/*queue type*/
	short 							ioTrap;						/*routine trap*/
	Ptr 							ioCmdAddr;					/*routine address*/
	ATPCompletionUPP 				ioCompletion;				/*ATPCompletionUPP or MPPCompletionUPP*/
	OSErr 							ioResult;					/*result code*/
	long 							userData;					/*Command result (ATP user bytes)*/
	short 							reqTID;						/*request transaction ID*/
	short 							ioRefNum;					/*driver reference number*/
	short 							csCode;						/*Call command code*/
	UInt8 							atpSocket;					/*currbitmap for requests or ATP socket number*/
	UInt8 							atpFlags;					/*control information*/
	AddrBlock 						addrBlock;					/*source/dest. socket address*/
	short 							reqLength;					/*request/response length*/
	Ptr 							reqPointer;					/*->request/response Data*/
	Ptr 							bdsPointer;					/*->response BDS */
	Ptr 							aKillQEl;					/*ptr to i/o queue element to cancel*/
};
typedef struct Killparms Killparms;

union ATPParamBlock {
	ATPparms 						ATP;						/*General ATP parms*/
	SendReqparms 					SREQ;						/*sendrequest parms*/
	ATPmisc1 						OTH1;						/*and a few others*/
	ATPmisc2 						OTH2;						/*and a few others*/
	Killparms 						KILL;						/*and a few others*/
};


#if GENERATINGCFM
#else
#endif

enum {
	uppATalkTransitionEventProcInfo = kCStackBased
		 | RESULT_SIZE(SIZE_CODE(sizeof(long)))
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(long)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(ATQEntryPtr)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(void *))),
	uppMPPCompletionProcInfo = kRegisterBased
		 | REGISTER_ROUTINE_PARAMETER(1, kRegisterA0, SIZE_CODE(sizeof(MPPPBPtr))),
	uppATPCompletionProcInfo = kRegisterBased
		 | REGISTER_ROUTINE_PARAMETER(1, kRegisterA0, SIZE_CODE(sizeof(ATPPBPtr))),
	uppXPPCompletionProcInfo = kRegisterBased
		 | REGISTER_ROUTINE_PARAMETER(1, kRegisterA0, SIZE_CODE(sizeof(XPPParmBlkPtr))),
	uppAttnRoutineProcInfo = kRegisterBased
		 | REGISTER_ROUTINE_PARAMETER(1, kRegisterD0, SIZE_CODE(sizeof(short)))
		 | REGISTER_ROUTINE_PARAMETER(2, kRegisterD1, SIZE_CODE(sizeof(short))),
	uppMPPProtocolHandlerProcInfo = SPECIAL_CASE_PROCINFO(7),
	uppDDPSocketListenerProcInfo = SPECIAL_CASE_PROCINFO(8)
};

#if GENERATINGCFM
#define NewATalkTransitionEventProc(userRoutine)		\
		(ATalkTransitionEventUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppATalkTransitionEventProcInfo, GetCurrentArchitecture())
#define NewMPPCompletionProc(userRoutine)		\
		(MPPCompletionUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppMPPCompletionProcInfo, GetCurrentArchitecture())
#define NewATPCompletionProc(userRoutine)		\
		(ATPCompletionUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppATPCompletionProcInfo, GetCurrentArchitecture())
#define NewXPPCompletionProc(userRoutine)		\
		(XPPCompletionUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppXPPCompletionProcInfo, GetCurrentArchitecture())
#define NewAttnRoutineProc(userRoutine)		\
		(AttnRoutineUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppAttnRoutineProcInfo, GetCurrentArchitecture())
#define NewMPPProtocolHandlerProc(userRoutine)		\
		(MPPProtocolHandlerUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppMPPProtocolHandlerProcInfo, GetCurrentArchitecture())
#define NewDDPSocketListenerProc(userRoutine)		\
		(DDPSocketListenerUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppDDPSocketListenerProcInfo, GetCurrentArchitecture())
#else
#define NewATalkTransitionEventProc(userRoutine)		\
		((ATalkTransitionEventUPP) (userRoutine))
#define NewMPPCompletionProc(userRoutine)		\
		((MPPCompletionUPP) (userRoutine))
#define NewATPCompletionProc(userRoutine)		\
		((ATPCompletionUPP) (userRoutine))
#define NewXPPCompletionProc(userRoutine)		\
		((XPPCompletionUPP) (userRoutine))
#define NewAttnRoutineProc(userRoutine)		\
		((AttnRoutineUPP) (userRoutine))
#define NewMPPProtocolHandlerProc(userRoutine)		\
		((MPPProtocolHandlerUPP) (userRoutine))
#define NewDDPSocketListenerProc(userRoutine)		\
		((DDPSocketListenerUPP) (userRoutine))
#endif

#if GENERATINGCFM
#define CallATalkTransitionEventProc(userRoutine, eventCode, qElem, eventParameter)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppATalkTransitionEventProcInfo, (eventCode), (qElem), (eventParameter))
#define CallMPPCompletionProc(userRoutine, thePBptr)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppMPPCompletionProcInfo, (thePBptr))
#define CallATPCompletionProc(userRoutine, thePBptr)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppATPCompletionProcInfo, (thePBptr))
#define CallXPPCompletionProc(userRoutine, thePBptr)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppXPPCompletionProcInfo, (thePBptr))
#define CallAttnRoutineProc(userRoutine, sessRefnum, attnBytes)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppAttnRoutineProcInfo, (sessRefnum), (attnBytes))
#define CallMPPProtocolHandlerProc(userRoutine, SCCAddr1, SCCAddr2, MPPLocalVars, nextFreeByteInRHA, ReadPacketAndReadRestPtr, numBytesLeftToReadInPacket)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppMPPProtocolHandlerProcInfo, (SCCAddr1), (SCCAddr2), (MPPLocalVars), (nextFreeByteInRHA), (ReadPacketAndReadRestPtr), (numBytesLeftToReadInPacket))
#define CallDDPSocketListenerProc(userRoutine, SCCAddr1, SCCAddr2, MPPLocalVars, nextFreeByteInRHA, ReadPacketAndReadRestPtr, packetDestinationNumber, numBytesLeftToReadInPacket)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppDDPSocketListenerProcInfo, (SCCAddr1), (SCCAddr2), (MPPLocalVars), (nextFreeByteInRHA), (ReadPacketAndReadRestPtr), (packetDestinationNumber), (numBytesLeftToReadInPacket))
#else
#define CallATalkTransitionEventProc(userRoutine, eventCode, qElem, eventParameter)		\
		(*(userRoutine))((eventCode), (qElem), (eventParameter))
/* (*MPPCompletionUPP) cannot be called from a high-level language without the Mixed Mode Manager */
/* (*ATPCompletionUPP) cannot be called from a high-level language without the Mixed Mode Manager */
/* (*XPPCompletionUPP) cannot be called from a high-level language without the Mixed Mode Manager */
/* (*AttnRoutineUPP) cannot be called from a high-level language without the Mixed Mode Manager */
/* (*MPPProtocolHandlerUPP) cannot be called from a high-level language without the Mixed Mode Manager */
/* (*DDPSocketListenerUPP) cannot be called from a high-level language without the Mixed Mode Manager */
#endif
extern pascal OSErr OpenXPP(short *xppRefnum);

extern pascal OSErr ASPOpenSession(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr ASPCloseSession(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr ASPAbortOS(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr ASPGetParms(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr ASPCloseAll(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr ASPUserWrite(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr ASPUserCommand(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr ASPGetStatus(XPPParmBlkPtr thePBptr, Boolean async);

#define ASPGetStatusSync(paramBlock) ASPGetStatus((paramBlock), FALSE)
extern pascal OSErr AFPCommand(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr GetLocalZones(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr GetZoneList(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr GetMyZone(XPPParmBlkPtr thePBptr, Boolean async);

extern pascal OSErr PAttachPH(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PDetachPH(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PWriteLAP(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr POpenSkt(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PCloseSkt(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PWriteDDP(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PRegisterName(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PLookupName(MPPPBPtr thePBptr, Boolean async);

#define PLookupNameSync(paramBlock) PLookupName((paramBlock), FALSE)
extern pascal OSErr PConfirmName(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PRemoveName(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PSetSelfSend(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PKillNBP(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PGetAppleTalkInfo(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr PATalkClosePrep(MPPPBPtr thePBptr, Boolean async);

extern pascal OSErr POpenATPSkt(ATPPBPtr thePBptr, Boolean async);

extern pascal OSErr PCloseATPSkt(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PSendRequest(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PGetRequest(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PSendResponse(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PAddResponse(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PRelTCB(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PRelRspCB(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PNSendRequest(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PKillSendReq(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr PKillGetReq(ATPPBPtr thePBPtr, Boolean async);

extern pascal OSErr ATPKillAllGetReq(ATPPBPtr thePBPtr, Boolean async);

extern pascal void BuildLAPwds(Ptr wdsPtr, Ptr dataPtr, short destHost, short prototype, short frameLen);

extern pascal void BuildDDPwds(Ptr wdsPtr, Ptr headerPtr, Ptr dataPtr, AddrBlock netAddr, short ddpType, short dataLen);

extern pascal void NBPSetEntity(Ptr buffer, ConstStr32Param nbpObject, ConstStr32Param nbpType, ConstStr32Param nbpZone);

extern pascal void NBPSetNTE(Ptr ntePtr, ConstStr32Param nbpObject, ConstStr32Param nbpType, ConstStr32Param nbpZone, short socket);

extern pascal short GetBridgeAddress(void );

extern pascal short BuildBDS(Ptr buffPtr, Ptr bdsPtr, short buffSize);

extern pascal OSErr MPPOpen(void );

extern pascal OSErr LAPAddATQ(ATQEntryPtr theATQEntry);

extern pascal OSErr LAPRmvATQ(ATQEntryPtr theATQEntry);

extern pascal OSErr ATPLoad(void );

extern pascal OSErr ATPUnload(void );

extern pascal OSErr NBPExtract(Ptr theBuffer, short numInBuf, short whichOne, EntityName *abEntity, AddrBlock *address);

extern pascal OSErr GetNodeAddress(short *myNode, short *myNet);

extern pascal Boolean IsMPPOpen(void );

extern pascal Boolean IsATPOpen(void );

extern pascal void ATEvent(long event, Ptr infoPtr);

extern pascal OSErr ATPreFlightEvent(long event, long cancel, Ptr infoPtr);

/*
	The following routines are obsolete and will not be supported on
	PowerPC. Equivalent functionality is provided by the routines
	above.
*/
#if OLDROUTINENAMES && !GENERATINGCFM
extern pascal OSErr MPPClose(void );

extern pascal OSErr LAPOpenProtocol(ABByte theLAPType, Ptr protoPtr);

extern pascal OSErr LAPCloseProtocol(ABByte theLAPType);

extern pascal OSErr LAPWrite(ATLAPRecHandle abRecord, Boolean async);

extern pascal OSErr LAPRead(ATLAPRecHandle abRecord, Boolean async);

extern pascal OSErr LAPRdCancel(ATLAPRecHandle abRecord);

extern pascal OSErr DDPOpenSocket(short *theSocket, Ptr sktListener);

extern pascal OSErr DDPCloseSocket(short theSocket);

extern pascal OSErr DDPRead(ATDDPRecHandle abRecord, Boolean retCksumErrs, Boolean async);

extern pascal OSErr DDPWrite(ATDDPRecHandle abRecord, Boolean doChecksum, Boolean async);

extern pascal OSErr DDPRdCancel(ATDDPRecHandle abRecord);

extern pascal OSErr ATPOpenSocket(AddrBlock addrRcvd, short *atpSocket);

extern pascal OSErr ATPCloseSocket(short atpSocket);

extern pascal OSErr ATPSndRequest(ATATPRecHandle abRecord, Boolean async);

extern pascal OSErr ATPRequest(ATATPRecHandle abRecord, Boolean async);

extern pascal OSErr ATPReqCancel(ATATPRecHandle abRecord, Boolean async);

extern pascal OSErr ATPGetRequest(ATATPRecHandle abRecord, Boolean async);

extern pascal OSErr ATPSndRsp(ATATPRecHandle abRecord, Boolean async);

extern pascal OSErr ATPAddRsp(ATATPRecHandle abRecord);

extern pascal OSErr ATPResponse(ATATPRecHandle abRecord, Boolean async);

extern pascal OSErr ATPRspCancel(ATATPRecHandle abRecord, Boolean async);

extern pascal OSErr NBPRegister(ATNBPRecHandle abRecord, Boolean async);

extern pascal OSErr NBPLookup(ATNBPRecHandle abRecord, Boolean async);

extern pascal OSErr NBPConfirm(ATNBPRecHandle abRecord, Boolean async);

extern pascal OSErr NBPRemove(EntityPtr abEntity);

extern pascal OSErr NBPLoad(void );

extern pascal OSErr NBPUnload(void );

#endif
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#ifdef __cplusplus
}
#endif

#endif /* __APPLETALK__ */

